#define analysis12_cxx
#include "analysis12.h"
#include <TH2.h>
#include <TStyle.h>
#include <TCanvas.h>

void analysis12::Loop()
{
//   In a ROOT session, you can do:
//      root> .L analysis12.C
//      root> analysis12 t
//      root> t.GetEntry(12); // Fill t data members with entry number 12
//      root> t.Show();       // Show values of entry 12
//      root> t.Show(16);     // Read and show values of entry 16
//      root> t.Loop();       // Loop on all entries
//

//     This is the loop skeleton where:
//    jentry is the global entry number in the chain
//    ientry is the entry number in the current Tree
//  Note that the argument to GetEntry must be:
//    jentry for TChain::GetEntry
//    ientry for TTree::GetEntry and TBranch::GetEntry
//
//       To read only selected branches, Insert statements like:
// METHOD1:
//    fChain->SetBranchStatus("*",0);  // disable all branches
//    fChain->SetBranchStatus("branchname",1);  // activate branchname
// METHOD2: replace line
//    fChain->GetEntry(jentry);       //read all branches
//by  b_branchname->GetEntry(ientry); //read only this branch
    fChain->SetBranchStatus("*",0);  // disable all branches
	fChain->SetBranchStatus("mcWeight",1);  // activate branchname
	fChain->SetBranchStatus("scaleFactor_ELE",1);  // activate branchname
	fChain->SetBranchStatus("scaleFactor_MUON",1);  // activate branchname
	fChain->SetBranchStatus("scaleFactor_LepTRIGGER",1);  // activate branchname
	fChain->SetBranchStatus("scaleFactor_PILEUP",1);  // activate branchname
 	fChain->SetBranchStatus("trigE",1);  // activate branchname
	fChain->SetBranchStatus("trigM",1);  // activate branchname
	fChain->SetBranchStatus("lep_n",1);  // activate branchname  
	fChain->SetBranchStatus("lep_pt",1);  // activate branchname  
	fChain->SetBranchStatus("lep_eta",1);  // activate branchname  
	fChain->SetBranchStatus("lep_phi",1);  // activate branchname  
	fChain->SetBranchStatus("lep_charge",1);  // activate branchname  
	fChain->SetBranchStatus("lep_type",1);  // activate branchname  
 	fChain->SetBranchStatus("lep_isTightID",1);  // activate branchname  
	fChain->SetBranchStatus("lep_ptcone30",1);  // activate branchname  
	fChain->SetBranchStatus("lep_etcone20",1);  // activate branchname  
	fChain->SetBranchStatus("lep_trackd0pvunbiased",1);  // activate branchname  
	fChain->SetBranchStatus("lep_tracksigd0pvunbiased",1);  // activate branchname 
	fChain->SetBranchStatus("jet_n",1);  // activate branchname  	
	fChain->SetBranchStatus("jet_pt",1);  // activate branchname  
	fChain->SetBranchStatus("SumWeights",1);  // activate branchname
	fChain->SetBranchStatus("XSection",1);  // activate branchname


// At this point we declare the output root file with the histograms we need
    TFile *rfile  = new TFile("OutputHistograms.root","RECREATE");
    TH1F *h_pt_mu = new TH1F("h_pt_mu","Muon pT ", 100,0.,100.);
    TH1F *h_eta_mu = new TH1F("h_eta_mu","Muon eta ", 54,-2.7,2.7);
    TH1F *h_phi_mu = new TH1F("h_phi_mu","Muon phi ", 50,-acos(-1.),acos(-1.));
    TH1F *h_pt_el = new TH1F("h_pt_el","Electron pT ", 100,0.,100.);
    TH1F *h_eta_el = new TH1F("h_eta_el","Electron eta ", 50,-2.5,2.5);
    TH1F *h_phi_el = new TH1F("h_phi_el","Electron phi ", 50,-acos(-1.),acos(-1.));
    TH1F *h_mll_mu = new TH1F("h_mll_mu","Invariant mass mu+mu-", 200,0,200.);
    TH1F *h_mll_el = new TH1F("h_mll_el","Invariant mass e+e- ", 200,0.,200.);

	double efficiency_el=0.;
	double efficiency_mu=0.;

   if (fChain == 0) return;

   Long64_t nentries = fChain->GetEntriesFast();

   Long64_t nbytes = 0, nb = 0;
   for (Long64_t jentry=0; jentry<nentries;jentry++) {	// run on all data
 //  for (Long64_t jentry=0; jentry<10000000;jentry++) {   // run on 1000000 events

      Long64_t ientry = LoadTree(jentry);
      if (ientry < 0) break;
      nb = fChain->GetEntry(jentry);   nbytes += nb;
      // if (Cut(ientry) < 0) continue;
// Show analysis progress
    if(jentry%10000==0)std::cout << "Analysed a total of: " << jentry << " events" << std::endl;


// At this point we construct the variables we need and apply our criteria
// As an example we loop over all leptons in the event 
// and we fill histograms with their pT (electrons and muons separately) if their pT exceeds 25 GeV 
// All quantities in the tree are expressed in MeV
		float MeV2GeV = 0.001;
//Scale factors to improve MC simulation with actual detector real data agreement
	  Float_t m_mcWeight = mcWeight;
	  //cout << m_mcWeight <<" "<< XSection<<" "<<SumWeights<<endl;
		float scaleFactor = scaleFactor_ELE*scaleFactor_LepTRIGGER*scaleFactor_PILEUP; 
//		float scaleFactor = scaleFactor_MUON*scaleFactor_LepTRIGGER*scaleFactor_PILEUP; 
		float weight =1.;  
//Total weight for MC event  = MC event weight  mcWeight * scale factor  
		float Luminosity=10000.;   
		if(mcWeight!=0.)weight = scaleFactor*(mcWeight/SumWeights)*(XSection*Luminosity);	 // weight used to fill all MC histograms 

// Here we impose selection criteria
	  if(!trigE)continue;   // Trigger for electrons
	  //if(!trigM)continue;   // Trigger for muons
        if(lep_n!=2)continue;
		if(lep_pt->at(0)*MeV2GeV<20. || lep_pt->at(1)*MeV2GeV<20.)continue;
		if(lep_charge->at(0)*lep_charge->at(1)>=0)continue; 
// ... and we fill our histograms
		if(lep_type->at(0)==11  && lep_type->at(1)==11)
		   {
			   h_pt_el->Fill((lep_pt->at(0))*MeV2GeV, weight);
			   h_pt_el->Fill((lep_pt->at(1))*MeV2GeV, weight);
			   h_eta_el->Fill(lep_eta->at(0), weight);
			   h_eta_el->Fill(lep_eta->at(1), weight);
			   h_phi_el->Fill(lep_phi->at(0), weight);
			   h_phi_el->Fill(lep_phi->at(1), weight);
			   efficiency_el+=mcWeight/SumWeights;
		   }

		if(lep_type->at(0)==13 && lep_type->at(1)==13)
		   {
			   h_pt_mu->Fill(lep_pt->at(0)*MeV2GeV, weight);
			   h_pt_mu->Fill(lep_pt->at(1)*MeV2GeV, weight);
			   h_eta_mu->Fill(lep_eta->at(0), weight);
			   h_eta_mu->Fill(lep_eta->at(1), weight);			   
			   h_phi_mu->Fill(lep_phi->at(0), weight);
			   h_phi_mu->Fill(lep_phi->at(1), weight);
			   efficiency_mu+=mcWeight/SumWeights;		   
			}

   }
   

	cout<< efficiency_el << " " << efficiency_mu<<endl; 

// At this point we write our histograms to the file and we close the file to finish
	h_pt_mu->Write();
	h_eta_mu->Write();
	h_phi_mu->Write();
	h_pt_el->Write();
	h_eta_el->Write();
	h_phi_el->Write();
	h_mll_mu->Write();
	h_mll_el->Write();
	rfile->Close();
//------------------------------------------------------------------------------



}
